import csv
import os.path
import subprocess
import sys
from typing import Dict, Iterable

import pytest

BASE_DIR = "/var/lib/id-broker-performance-tests"
VENV = os.path.join(BASE_DIR, "venv")
LOCUST_EXE = os.path.join(VENV, "bin", "locust")
ENV_KEYCLOAK_HOST = "UCS_ENV_KEYCLOAK_FQDN"
ENV_PROVISIONING_HOST = "UCS_ENV_PROVISIONING_FQDN"
ENV_PRIMARY_HOST = "UCS_ENV_PRIMARY_FQDN"
ENV_SELF_DISCLOSURE_HOST = "UCS_ENV_SELF_DISCLOSURE_FQDN"
ENV_TRAEGER1_HOST = "UCS_ENV_TRAEGER1_FQDN"
ENV_TRAEGER2_HOST = "UCS_ENV_TRAEGER2_FQDN"
NUM_USERS = "4"
SPAWN_RATE = "1"
RUNTIME = "20s"  # TODO: 5m
STOP_TIMEOUT = "60"


@pytest.fixture(scope="session")
def host_from_env():
    def _func(host: str) -> str:
        env_var = {
            "keycloak": ENV_KEYCLOAK_HOST,
            "provisioning": ENV_PROVISIONING_HOST,
            "primary": ENV_PRIMARY_HOST,
            "self-disclosure": ENV_SELF_DISCLOSURE_HOST,
            "traeger1": ENV_TRAEGER1_HOST,
            "traeger2": ENV_TRAEGER2_HOST,
        }[host]
        return os.environ[env_var]

    return _func


@pytest.fixture(scope="session")
def rows():
    def _func(csv_file_name: str) -> Iterable[Dict[str, str]]:
        print("Reading {!r}...".format(csv_file_name))
        with open(csv_file_name) as fp:
            yield from csv.DictReader(fp)

    return _func


@pytest.fixture(scope="session")
def get_one_row(rows):
    def _func(csv_file: str, column_name: str, column_value: str) -> Dict[str, str]:
        for row in rows(csv_file):
            if row[column_name] == column_value:
                return row
        raise ValueError(
            "No row found that had a column {!r} with value {!r}.".format(column_name, column_value)
        )

    return _func


@pytest.fixture(scope="session")
def check_failure_count(rows):
    def _func(result_file_base_path: str) -> None:
        csv_file = f"{result_file_base_path}_stats.csv"
        col = "Failure Count"
        for row in rows(csv_file):
            value = int(row[col])
            assert value == 0

    return _func


@pytest.fixture(scope="session")
def check_rps(get_one_row):
    def _func(result_file_base_path: str, url_name: str, expected_min: float) -> None:
        csv_file = f"{result_file_base_path}_stats.csv"
        row = get_one_row(csv_file, "Name", url_name)
        col = "Requests/s"
        value = float(row[col])
        assert value > expected_min

    return _func


@pytest.fixture(scope="session")
def check_95_percentile(get_one_row):
    def _func(result_file_base_path: str, url_name: str, expected_max: int) -> None:
        csv_file = f"{result_file_base_path}_stats.csv"
        row = get_one_row(csv_file, "Name", url_name)
        col = "95%"
        value = int(row[col])
        assert value < expected_max

    return _func


@pytest.fixture(scope="session")
def execute_test():
    def _func(locust_path: str, result_file_base_path: str, host: str, loglevel: str = "INFO") -> None:
        cmd = [
            LOCUST_EXE,
            "--loglevel",
            loglevel,
            "--locustfile",
            locust_path,
            "--host",
            host,
            "--headless",
            "--users",
            NUM_USERS,
            "--spawn-rate",
            SPAWN_RATE,
            "--run-time",
            RUNTIME,
            "--stop-timeout",
            STOP_TIMEOUT,
            f"--csv={result_file_base_path}",
            f"--html={result_file_base_path}.html",
            "--print-stats",
            "--exit-code-on-error",
            "0",
        ]
        print(f"Executing: {cmd!r}")
        process = subprocess.Popen(cmd, stdout=sys.stdout, stderr=sys.stdout)  # nosec
        process.communicate()

    return _func
